import numpy as np
import soundfile as sf
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('filename')
args = parser.parse_args()

RORQUAL_HOPSIZE, RORQUAL_WINSIZE, RORQUAL_FE, RORQUAL_SAMPLEPERSAMPLE, RORQUAL_LENSIG = 256, 4096, 4000, 25600, 60
CACHA_HOPSIZE, CACHA_WINSIZE, CACHA_FE, CACHA_SAMPLEPERSAMPLE, CACHA_LENSIG = 256, 512, 64000, 12800, 60
CACHA_RAPPORT_FE, RORQUAL_RAPPORT_FE, RAPPORT_N_CHANNELS = 128_000, 128_00, 5
CACHA_NUM_DETEC, RORQUAL_NUM_DETEC = 10, 3
rorqual_predPos = lambda p: int(((p * RORQUAL_HOPSIZE + (RORQUAL_WINSIZE/2 + RORQUAL_HOPSIZE * 13) / 2) / RORQUAL_FE - 1) * RORQUAL_RAPPORT_FE)
cacha_predPos = lambda p: int(((p * CACHA_HOPSIZE * 8 + (CACHA_WINSIZE + CACHA_HOPSIZE * 7 * 6) / 4 ) / CACHA_FE - 0.05) * CACHA_RAPPORT_FE)

file = open(args.filename, 'r')
lines = iter(file.readlines())

for l in lines:
    if l == ' rorqual predPeaks\n':
        l = next(lines)
        if ',' in l:
            sig = np.zeros((RORQUAL_LENSIG * RORQUAL_RAPPORT_FE, RAPPORT_N_CHANNELS), dtype='int16')
            peaks = np.array(l[:-2].split(',')).astype(int)
            l = next(lines)
            for p in peaks[:RORQUAL_NUM_DETEC]:
                l = next(lines)
                samples = np.array(l[:-2].split(',')).astype('int16').reshape(-1, RAPPORT_N_CHANNELS)
                sig[rorqual_predPos(p):rorqual_predPos(p)+RORQUAL_SAMPLEPERSAMPLE] = samples[:min(rorqual_predPos(p)+RORQUAL_SAMPLEPERSAMPLE, len(sig))-rorqual_predPos(p)]
            sf.write(f'{args.filename[:-8]}_rorqual.wav', sig, RORQUAL_RAPPORT_FE)
    if l == ' cacha predPeaks\n':
        l = next(lines)
        if ',' in l:
            sig = np.zeros((CACHA_LENSIG * CACHA_RAPPORT_FE, RAPPORT_N_CHANNELS), dtype='int16')
            peaks = np.array(l[:-2].split(',')).astype(int)
            l = next(lines)
            for p in peaks[:CACHA_NUM_DETEC]:
                l = next(lines)
                samples = np.array(l[:-2].split(',')).astype('int16').reshape(-1, RAPPORT_N_CHANNELS)
                sig[cacha_predPos(p):cacha_predPos(p)+CACHA_SAMPLEPERSAMPLE] = samples
            sf.write(f'{args.filename[:-8]}_cacha.wav', sig, CACHA_RAPPORT_FE)

file.close()
